import { Completer } from '../../../../domain/entities/engine/processor/completer/completer.abstract';
import { MatchRequest } from '../../../../domain/dtos/match.request';
import { Candidate } from '../../../../domain/entities/engine/candidate';
import { AlgorithmType } from '../../../../domain/types/algorithm.enum';
import { IDefaultParams } from '../../../../domain/types/default-params.type';
import { MatchQuery } from '../../../../queries/match.query';

const mockGeorouterCreator = {
  create: jest.fn().mockImplementation(),
};

const mockTimezoneFinder = {
  timezones: jest.fn().mockImplementation(),
};

const defaultParams: IDefaultParams = {
  DEFAULT_UUID: '00000000-0000-0000-0000-000000000000',
  MARGIN_DURATION: 900,
  VALIDITY_DURATION: 365,
  DEFAULT_TIMEZONE: 'Europe/Paris',
  DEFAULT_SEATS: 3,
  DEFAULT_ALGORITHM_SETTINGS: {
    algorithm: AlgorithmType.CLASSIC,
    strict: false,
    remoteness: 15000,
    useProportion: true,
    proportion: 0.3,
    useAzimuth: true,
    azimuthMargin: 10,
    maxDetourDistanceRatio: 0.3,
    maxDetourDurationRatio: 0.3,
    georouterType: 'graphhopper',
    georouterUrl: 'http://localhost',
  },
};

const matchRequest: MatchRequest = new MatchRequest();
matchRequest.departure = '2023-04-01 12:00';
matchRequest.waypoints = [
  {
    lat: 49.440041,
    lon: 1.093912,
  },
  {
    lat: 50.630992,
    lon: 3.045432,
  },
];
const matchQuery: MatchQuery = new MatchQuery(
  matchRequest,
  defaultParams,
  mockGeorouterCreator,
  mockTimezoneFinder,
);

class FakeCompleter extends Completer {
  complete = (candidates: Candidate[]): Candidate[] => {
    return candidates;
  };
}

describe('Completer', () => {
  it('should create an extended class', () => {
    expect(new FakeCompleter(matchQuery)).toBeDefined();
  });

  it('should call complete method', () => {
    const fakeCompleter: Completer = new FakeCompleter(matchQuery);
    const completerSpy = jest.spyOn(fakeCompleter, 'complete');
    fakeCompleter.execute([new Candidate()]);
    expect(completerSpy).toHaveBeenCalled();
  });
});
